{% extends "defindex.html" %}
{% block tables %}
<p><strong>For users:</strong></p>
<table class="contentstable" align="center"><tr>
    <td width="50%">
      <p class="biglink"><a class="biglink" href="{{ pathto("user/setting-up") }}">Setting Up</a><br/>
	<span class="linkdescr">Learn about what NumPy is and how to install it</span></p>
      <p class="biglink"><a class="biglink" href="{{ pathto("user/quickstart") }}">Quickstart Tutorial</a><br/>
	<span class="linkdescr">Aimed at domain experts or people migrating to NumPy</span></p>
      <p class="biglink"><a class="biglink" href="{{ pathto("user/absolute_beginners") }}">Absolute Beginners Tutorial</a><br/>
	<span class="linkdescr">Start here for an overview of NumPy features and syntax</span></p>
      <p class="biglink"><a class="biglink" href="{{ pathto("user/tutorials_index") }}">Tutorials</a><br/>
	<span class="linkdescr">Learn about concepts and submodules</span></p>
      <p class="biglink"><a class="biglink" href="{{ pathto("user/howtos_index") }}">How Tos</a><br/>
	<span class="linkdescr">How to do common tasks with NumPy</span></p>
      <p class="biglink"><a class="biglink" href="{{ pathto("reference/index") }}">NumPy API Reference</a><br/>
        <span class="linkdescr">Automatically generated reference documentation</span></p>
      <p class="biglink"><a class="biglink" href="{{ pathto("user/explanations_index") }}">Explanations</a><br/>
	<span class="linkdescr">In depth explanation of concepts, best practices and techniques</span></p>
      <p class="biglink"><a class="biglink" href="{{ pathto("f2py/index") }}">F2Py Guide</a><br/>
        <span class="linkdescr">Documentation for the f2py module (Fortran extensions for Python)</span></p>
      <p class="biglink"><a class="biglink" href="{{ pathto("glossary") }}">Glossary</a><br/>
        <span class="linkdescr">List of the most important terms</span></p>
  </td></tr>
</table>

<p><strong>For developers/contributors:</strong></p>
<table class="contentstable" align="center"><tr>
    <td width="50%">
      <p class="biglink"><a class="biglink" href="{{ pathto("dev/index") }}">NumPy Contributor Guide</a><br/>
        <span class="linkdescr">Contributing to NumPy</span></p>
      <p class="biglink"><a class="biglink" href="{{ pathto("dev/underthehood") }}">Under-the-hood docs</a><br/>
	<span class="linkdescr">Specialized, in-depth documentation</span></p>
      <p class="biglink"><a class="biglink" href="{{ pathto("docs/index") }}">Building and Extending the Documentation</a><br/>
        <span class="linkdescr">How to contribute to this documentation (user and API)</span></p>
      <p class="biglink"><a class="biglink" href="{{ pathto("docs/howto_document") }}">The numpydoc docstring guide</a><br/>
	<span class="linkdescr">How to write docstrings in the numpydoc format</span></p>
      <p class="biglink"><a class="biglink" href="{{ pathto("benchmarking") }}">Benchmarking</a><br/>
        <span class="linkdescr">benchmarking NumPy</span></p>
      <p class="biglink"><a class="biglink" href="https://www.numpy.org/neps/index.html">NumPy Enhancement Proposals</a><br/>
    </td></tr>
  </table>

  <p><strong>Meta information:</strong></p>
  <table class="contentstable" align="center"><tr>
    <td width="50%">
      <p class="biglink"><a class="biglink" href="{{ pathto("bugs") }}">Reporting bugs</a></p>
      <p class="biglink"><a class="biglink" href="{{ pathto("release") }}">Release Notes</a></p>
    </td><td width="50%">
      <p class="biglink"><a class="biglink" href="{{ pathto("about") }}">About NumPy</a></p>
      <p class="biglink"><a class="biglink" href="{{ pathto("license") }}">License of NumPy</a></p>
    </td></tr>
  </table>

  <h2>Acknowledgements</h2>
  <p>
    Large parts of this manual originate from Travis E. Oliphant's book
    <a href="https://archive.org/details/NumPyBook">"Guide to NumPy"</a>
    (which generously entered Public Domain in August 2008). The reference
    documentation for many of the functions are written by numerous
    contributors and developers of NumPy.
  </p>
  <p>
    The preferred way to update the documentation is by submitting a pull
    request on Github (see the <a href="{{ pathto("docs/index") }}">Documentation Index</a>).
    Please help us to further improve the NumPy documentation!
  </p>
{% endblock %}
