{% extends "!layout.html" %}

{%- block header %}
<div class="container">
  <div class="top-scipy-org-logo-header" style="background-color: #a2bae8;">
    <a href="{{ pathto('index') }}">
      <img border=0 alt="NumPy" src="{{ pathto('_static/numpy_logo.png', 1) }}"></a>
    </div>
  </div>
</div>

{% endblock %}
{% block rootrellink %}
        {% if pagename != 'index' %}
        <li class="active"><a href="{{ pathto('index') }}">{{ shorttitle|e }}</a></li>
        {% endif %}
{% endblock %}

{% block sidebarsearch %}
{%- if sourcename %}
<ul class="this-page-menu">
{%- if 'reference/generated' in sourcename %}
  <li><a href="/numpy/docs/{{ sourcename.replace('reference/generated/', '').replace('.txt', '') |e }}">{{_('Edit page')}}</a></li>
{%- else %}
  <li><a href="/numpy/docs/numpy-docs/{{ sourcename.replace('.txt', '.rst') |e }}">{{_('Edit page')}}</a></li>
{%- endif %}
</ul>
{%- endif %}
{{ super() }}
{% endblock %}
